#ifndef STAN_MATH_REV_ARR_HPP
#define STAN_MATH_REV_ARR_HPP

#include <stan/math/rev/core.hpp>
#include <stan/math/rev/scal/meta/is_var.hpp>
#include <stan/math/rev/scal/meta/partials_type.hpp>

#include <stan/math/prim/arr.hpp>
#include <stan/math/rev/scal.hpp>

#include <stan/math/rev/arr/fun/log_sum_exp.hpp>
#include <stan/math/rev/arr/fun/to_var.hpp>
#include <stan/math/rev/arr/functor/coupled_ode_system.hpp>
#include <stan/math/rev/arr/functor/integrate_1d.hpp>

#endif
