#ifndef BOOST_METAPARSE_LIMIT_ONE_OF_SIZE_HPP
#define BOOST_METAPARSE_LIMIT_ONE_OF_SIZE_HPP

// Copyright Abel Sinkovics (abel@sinkovics.hu)  2013.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_METAPARSE_LIMIT_ONE_OF_SIZE
#  define BOOST_METAPARSE_LIMIT_ONE_OF_SIZE 20
#endif

#endif

