#ifndef STAN_LANG_AST_NIL_HPP
#define STAN_LANG_AST_NIL_HPP

namespace stan {
  namespace lang {

    /**
     * The nil structure used as a placeholder for undefined or empty
     * values in several structures.
     */
    struct nil {
    };

  }
}
#endif
